@@ -51,6 +51,8 @@ typedef enum {
     XML_SCHEMAS_DATE,
     XML_SCHEMAS_DATETIME,
     XML_SCHEMAS_DURATION,
+    XML_SCHEMAS_FLOAT,
+    XML_SCHEMAS_DOUBLE,
     XML_SCHEMAS_,
     XML_SCHEMAS_XXX
 } xmlSchemaValType;
@@ -100,6 +102,8 @@ struct _xmlSchemaVal {
 	xmlSchemaValDecimal     decimal;
         xmlSchemaValDate        date;
         xmlSchemaValDuration    dur;
+	float			f;
+	double			d;
     } value;
 };
 
@@ -122,6 +126,8 @@ static xmlSchemaTypePtr xmlSchemaTypeDurationDef = NULL;
 static xmlSchemaTypePtr xmlSchemaTypePositiveIntegerDef = NULL;
 static xmlSchemaTypePtr xmlSchemaTypeNonNegativeIntegerDef = NULL;
 static xmlSchemaTypePtr xmlSchemaTypeNmtoken = NULL;
+static xmlSchemaTypePtr xmlSchemaTypeFloatDef = NULL;
+static xmlSchemaTypePtr xmlSchemaTypeDoubleDef = NULL;
 
 /*
  * xmlSchemaInitBasicType:
@@ -176,6 +182,8 @@ xmlSchemaInitTypes(void) {
     xmlSchemaTypeNonNegativeIntegerDef =
 	xmlSchemaInitBasicType("nonNegativeInteger");
     xmlSchemaTypeNmtoken = xmlSchemaInitBasicType("NMTOKEN");
+    xmlSchemaTypeFloatDef = xmlSchemaInitBasicType("float");
+    xmlSchemaTypeDoubleDef = xmlSchemaInitBasicType("double");
 
     xmlSchemaTypesInitialized = 1;
 }
@@ -660,8 +668,8 @@ _xmlSchemaParseTimeZone (xmlSchemaValDatePtr dt, const xmlChar **str) {
  *         and -1 in case of internal or API error.
  */
 static int
-xmlSchemaValidateDates (xmlSchemaTypePtr type, const xmlChar *dateTime,
-                        xmlSchemaValPtr *val) {
+xmlSchemaValidateDates (xmlSchemaTypePtr type ATTRIBUTE_UNUSED,
+	                const xmlChar *dateTime, xmlSchemaValPtr *val) {
     xmlSchemaValPtr dt;
     int ret;
     const xmlChar *cur = dateTime;
@@ -819,8 +827,8 @@ error:
  *         and -1 in case of internal or API error.
  */
 static int
-xmlSchemaValidateDuration (xmlSchemaTypePtr type, const xmlChar *duration,
-                        xmlSchemaValPtr *val) {
+xmlSchemaValidateDuration (xmlSchemaTypePtr type ATTRIBUTE_UNUSED,
+	                   const xmlChar *duration, xmlSchemaValPtr *val) {
     const xmlChar  *cur = duration;
     xmlSchemaValPtr dur;
     int isneg = 0;
@@ -1059,6 +1067,38 @@ xmlSchemaValidatePredefinedType(xmlSchemaTypePtr type, const xmlChar *value,
 	    }
 	}
 	return(0);
+    } else if (type == xmlSchemaTypeFloatDef) {
+	const xmlChar *cur = value, *tmp;
+	int frac = 0, len, neg = 0;
+	unsigned long base = 0;
+	if (cur == NULL)
+	    return(1);
+	if (*cur == '+')
+	    cur++;
+	else if (*cur == '-') {
+	    neg = 1;
+	    cur++;
+	}
+	tmp = cur;
+	while ((*cur >= '0') && (*cur <= '9')) {
+	    base = base * 10 + (*cur - '0');
+	    cur++;
+	}
+	len = cur - tmp;
+	if (*cur == '.') {
+	    cur++;
+	    tmp = cur;
+	    while ((*cur >= '0') && (*cur <= '9')) {
+		base = base * 10 + (*cur - '0');
+		cur++;
+	    }
+	    frac = cur - tmp;
+	}
+	TODO
+	return(0);
+    } else if (type == xmlSchemaTypeDoubleDef) {
+	TODO
+	return(0);
     } else {
 	TODO
 	return(0);
@@ -1463,18 +1503,25 @@ xmlSchemaCompareDates (xmlSchemaValPtr x, xmlSchemaValPtr y)
             q1 = xmlSchemaDateNormalize(y, (14 * SECS_PER_HOUR));
 
             q1d = _xmlSchemaDateCastYMToDays(q1) + q1->value.date.day;
-            if (p1d < q1d)
+            if (p1d < q1d) {
+		xmlSchemaFreeValue(p1);
+		xmlSchemaFreeValue(q1);
                 return -1;
-            else if (p1d == q1d) {
+	    } else if (p1d == q1d) {
                 double sec;
 
                 sec = TIME_TO_NUMBER(p1) - TIME_TO_NUMBER(q1);
-                if (sec < 0.0)
+                if (sec < 0.0) {
+		    xmlSchemaFreeValue(p1);
+		    xmlSchemaFreeValue(q1);
                     return -1;
-                else {
+		} else {
                     /* normalize y - 14:00 */
                     q2 = xmlSchemaDateNormalize(y, -(14 * SECS_PER_HOUR));
                     q2d = _xmlSchemaDateCastYMToDays(q2) + q2->value.date.day;
+		    xmlSchemaFreeValue(p1);
+		    xmlSchemaFreeValue(q1);
+		    xmlSchemaFreeValue(q2);
                     if (p1d > q2d)
                         return 1;
                     else if (p1d == q2d) {
@@ -1485,7 +1532,10 @@ xmlSchemaCompareDates (xmlSchemaValPtr x, xmlSchemaValPtr y)
                             return 2; /* indeterminate */
                     }
                 }
-            }
+            } else {
+		xmlSchemaFreeValue(p1);
+		xmlSchemaFreeValue(q1);
+	    }
         }
     } else if (y->value.date.tz_flag) {
         q1 = xmlSchemaDateNormalize(y, 0);
@@ -1495,19 +1545,26 @@ xmlSchemaCompareDates (xmlSchemaValPtr x, xmlSchemaValPtr y)
         p1 = xmlSchemaDateNormalize(x, -(14 * SECS_PER_HOUR));
         p1d = _xmlSchemaDateCastYMToDays(p1) + p1->value.date.day;
 
-        if (p1d < q1d)
+        if (p1d < q1d) {
+	    xmlSchemaFreeValue(p1);
+	    xmlSchemaFreeValue(q1);
             return -1;
-        else if (p1d == q1d) {
+	} else if (p1d == q1d) {
             double sec;
 
             sec = TIME_TO_NUMBER(p1) - TIME_TO_NUMBER(q1);
-            if (sec < 0.0)
+            if (sec < 0.0) {
+		xmlSchemaFreeValue(p1);
+		xmlSchemaFreeValue(q1);
                 return -1;
-            else {
+	    } else {
                 /* normalize x + 14:00 */
                 p2 = xmlSchemaDateNormalize(x, (14 * SECS_PER_HOUR));
                 p2d = _xmlSchemaDateCastYMToDays(p2) + p2->value.date.day;
 
+		xmlSchemaFreeValue(p1);
+		xmlSchemaFreeValue(q1);
+		xmlSchemaFreeValue(p2);
                 if (p2d > q1d)
                     return 1;
                 else if (p2d == q1d) {
@@ -1518,6 +1575,9 @@ xmlSchemaCompareDates (xmlSchemaValPtr x, xmlSchemaValPtr y)
                         return 2; /* indeterminate */
                 }
             }
+	} else {
+	    xmlSchemaFreeValue(p1);
+	    xmlSchemaFreeValue(q1);
         }
     }
 
@@ -1531,14 +1591,20 @@ xmlSchemaCompareDates (xmlSchemaValPtr x, xmlSchemaValPtr y)
         p1 = xmlSchemaDateNormalize(x, 0);
         p1d = _xmlSchemaDateCastYMToDays(p1) + p1->value.date.day;
 
-        if (p1d < q1d)
+        if (p1d < q1d) {
+	    xmlSchemaFreeValue(p1);
+	    xmlSchemaFreeValue(q1);
             return -1;
-        else if (p1d > q1d)
+	} else if (p1d > q1d) {
+	    xmlSchemaFreeValue(p1);
+	    xmlSchemaFreeValue(q1);
             return 1;
-        else {
+	} else {
             double sec;
 
             sec = TIME_TO_NUMBER(p1) - TIME_TO_NUMBER(q1);
+	    xmlSchemaFreeValue(p1);
+	    xmlSchemaFreeValue(q1);
             if (sec < 0.0)
                 return -1;
             else if (sec > 0.0)
@@ -1726,7 +1792,8 @@ xmlSchemaCompareValues(xmlSchemaValPtr x, xmlSchemaValPtr y) {
  *     number otherwise and -1 in case of internal or API error.
  */
 int
-xmlSchemaValidateFacet(xmlSchemaTypePtr base, xmlSchemaFacetPtr facet,
+xmlSchemaValidateFacet(xmlSchemaTypePtr base ATTRIBUTE_UNUSED,
+	               xmlSchemaFacetPtr facet,
 	               const xmlChar *value, xmlSchemaValPtr val)
 {
     int ret;
@@ -1788,7 +1855,7 @@ xmlSchemaValidateFacet(xmlSchemaTypePtr base, xmlSchemaFacetPtr facet,
 	    if ((facet->val != NULL) &&
 		(facet->val->type == XML_SCHEMAS_DECIMAL) &&
 		(facet->val->value.decimal.frac == 0)) {
-		int len;
+		unsigned int len;
 
 		if (facet->val->value.decimal.sign == 1)
 		    return(1);
